from typing import Dict, List

from .event import Event
from .event_listener import EventListener


class EventBus:
    __listeners: Dict[str, List[EventListener]]

    def __init__(self):
        self.__listeners = {}

    def listen(self, listener: EventListener):
        event_name = listener.get_event_name()
        if event_name not in self.__listeners:
            self.__listeners[event_name] = []

        self.__listeners[event_name].append(listener)

    async def dispatch(self, event: Event):
        if event.get_name() not in self.__listeners:
            return

        listeners = self.__listeners[event.get_name()]
        for listener in listeners:
            await listener(event)



from abc import ABC, abstractmethod
from typing import Generic, TypeVar, get_args

from .event import Event
from .event_listener import EventListener

T = TypeVar("T", bound=Event)


class TypedEventListener(EventListener, Generic[T], ABC):
    @abstractmethod
    async def __call__(self, event: T):
        ...

    def get_event_name(self) -> str:
        type_args = get_args(self.__class__.__orig_bases__[0])
        return type_args[0].__name__



import pytest

from oop_bus import Event, EventBus, TypedEventListener


class TestEvent(Event):
    ...


class EventListenerForTest(TypedEventListener[TestEvent]):
    def __init__(self):
        self.called = 0

    async def __call__(self, event: TestEvent):
        self.called += 1


class TestEventBus:
    @pytest.mark.asyncio
    async def test_it_should_run_listeners_for_event(self):
        bus = EventBus()
        event = TestEvent()

        listener1 = EventListenerForTest()
        listener2 = EventListenerForTest()

        bus.listen(listener1)
        bus.listen(listener2)

        await bus.dispatch(event)

        assert listener1.called == 1
        assert listener2.called == 1

    @pytest.mark.asyncio
    async def test_it_should_not_run_listener_if_event_differs(self):
        bus = EventBus()
        event = Event()

        listener1 = EventListenerForTest()
        listener2 = EventListenerForTest()

        bus.listen(listener1)
        bus.listen(listener2)

        await bus.dispatch(event)

        assert listener1.called == 0
        assert listener2.called == 0



from .event import Event
from .event_listener import EventListener
from .event_bus import EventBus
from.typed_event_listener import TypedEventListener

__all__ = ["Event", "EventBus", "EventListener", "TypedEventListener"]



from abc import ABC, abstractmethod

from .event import Event


class EventListener(ABC):
    @abstractmethod
    async def __call__(self, event: Event):
        ...

    @abstractmethod
    def get_event_name(self) -> str:
        ...
